/*
 * $Id: DeltaProcessor.java,v 1.7 2005/01/12 16:06:48 dallmeier Exp $
 * 
 * LICENSE:
 */
package org.softevo.rose.assist;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.Vector;

import org.eclipse.compare.HistoryItem;
import org.eclipse.compare.ResourceNode;
import org.eclipse.compare.structuremergeviewer.IStructureCreator;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFileState;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.IResourceDeltaVisitor;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.team.internal.ccvs.core.resources.CVSWorkspaceRoot;
import org.softevo.rose.compare.ModificationItem;
import org.softevo.rose.compare.RoseComparePlugin;

/** This is the resource visitor that identifies
 *  the resources that changed. <BR>
 *  It is invoked by eclipse whenever a resource
 *  changed.
 */
public class DeltaProcessor implements IResourceDeltaVisitor { 
  
  /** The set of changed entities. <BR>
   */
  private Vector changedEntities = null;
  
  /** Creates a new delta processor. <BR>
   */
  public DeltaProcessor() {
    changedEntities = new Vector();
  }

  /** Tests if this processor found
   *  changed entities. <BR>
   */
  public boolean hasChangedEntities() {
    return !changedEntities.isEmpty();
  }
  
  /** Gets the list of changed entities. <BR>
   */
  public Vector getChangedEntities() {
    return changedEntities;
  }
  
  /** Visits the delta. <BR>
   *  If the changed resource was a file, this method
   *  tries to retrieve the last version from the history.
   *  These two entries are compared to find the changed
   *  entities. 
   */
  public boolean visit(IResourceDelta delta) throws CoreException {
    IResource resource;
    IFile file;
    IFileState lastHistoryState;
    HistoryItem historyItem;
    IStructureCreator structureCreator;
    LinkedList modificationItems;
    Iterator iter;
    
    resource = delta.getResource();
    if (resource instanceof IFile) {
        file = (IFile) resource;
        if (!CVSWorkspaceRoot.isSharedWithCVS(file)) {
            return false; //ignore unshared files
        }
        if (".rose.tmp".equals(file.getProjectRelativePath().segment(0))) {
            return false; //ignore rose temp files
        }
        structureCreator = RoseComparePlugin.getStructureCreator(file.getFileExtension());
        if (structureCreator != null) {
            lastHistoryState = getLastHistoryElement(file);
            if (lastHistoryState != null) {
                historyItem = new HistoryItem(new ResourceNode(file), lastHistoryState);
                modificationItems = RoseComparePlugin.doStructureCompare(structureCreator, historyItem, new ResourceNode(file));
                iter = modificationItems.iterator();
                while (iter.hasNext()) {
                    ModificationItem item = (ModificationItem) iter.next();
                    String remoteFileName = "/" + CVSWorkspaceRoot.getCVSFileFor(file).getRepositoryRelativePath();
                    item.setAttachedFileName(remoteFileName); 
                    if (!changedEntities.contains(item)) {
                        changedEntities.add(item);
                    } 
                }
            }
        } else { // we don't have a structure creator
            int kind = -1;
            switch (delta.getKind()) {
            case IResourceDelta.ADDED:
                kind = ModificationItem.ADDED;
                break;
            case IResourceDelta.CHANGED:
                kind = ModificationItem.CHANGED;                
                break;
            case IResourceDelta.REMOVED:
                kind = ModificationItem.DELETED;
                break;
            default:
                break;
            }
            if (kind==ModificationItem.CHANGED) {
                ModificationItem item = new ModificationItem(new ResourceNode(file), kind);
                item.setAttachedFileName(file.getFullPath().toString());
                if (!changedEntities.contains(item)) {
                    changedEntities.add(item);
                }
            }
        }
        return false;
    }
    return CVSWorkspaceRoot.isSharedWithCVS(resource);
  }  
  
  /** Gets the last file state from the history
   *  of the file passed. <BR>
   *  @param file the file
   *  @return the last history state or <code>null</code>
   */
  private IFileState getLastHistoryElement(IFile file) {
    
    // Get last element in local history
    try {
      IFileState[] ifs = file.getHistory(new NullProgressMonitor());
      
      if (ifs.length < 2) {
        // There is only one version (the current) in the history
        return null;
      }
      
      // Get the version before file
      // TODO Is it guaranteed that ifs is ordered by modification date?
      int pos = 0;
      long max = -1;
      for (int i = 0; i < ifs.length; i++) {
        //System.out.println("X: " + ifs[i].getModificationTime());
        if (ifs[i].getModificationTime() > max) {
          max = ifs[i].getModificationTime();
          pos = i;
        }
      }
      
      return ifs[pos];
    } catch (CoreException e) {
      return null;
    }
  }
  
  
}
