/*
 * $Id: PreprocessingRosePlugin.java,v 1.12.2.1 2005/04/09 12:43:48 zimmerth Exp $
 * 
 * LICENSE:
 */
package org.softevo.rose.assist.preprocessing;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.IJobManager;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.progress.IProgressConstants;
import org.softevo.rose.AbstractRoseCVSPlugin;
import org.softevo.rose.IRoseCVSPlugin;
import org.softevo.rose.IRosePlugin;
import org.softevo.rose.RosePluginException;
import org.softevo.rose.assist.AssistPlugin;
import org.softevo.rose.assist.mining.SetupRoseJob;
import org.softevo.rose.assist.mining.SetupRosePlugin;
import org.softevo.rose.ccvs.IRepositoryProvider;
import org.softevo.rose.ccvs.RepositoryException;
import org.softevo.rose.ccvs.eclipse.EclipseRepositoryProvider;
import org.softevo.rose.data.entities.EntitiesJob;
import org.softevo.rose.data.entities.EntitiesRosePlugin;
import org.softevo.rose.data.mirror.ExtractJob;
import org.softevo.rose.data.mirror.ExtractRosePlugin;
import org.softevo.rose.data.mirror.GroupJob;
import org.softevo.rose.data.mirror.GroupRosePlugin;
import org.softevo.rose.db.DatabaseProvider;

/**
 * @author Thomas Zimmermann, zimmerth@cs.uni-sb.de
 */
public class PreprocessingRosePlugin extends AbstractRoseCVSPlugin {

    private IRosePlugin[] plugins = null;
    private int[] ticks = { ExtractJob.WORK, GroupJob.WORK, 
            EntitiesJob.WORK, SetupRoseJob.WORK };
    
    /**
     * @param db
     * @param cvs
     */
    public PreprocessingRosePlugin() {
        super();
        IRosePlugin[] p = {
                new ExtractRosePlugin(),
                new GroupRosePlugin(),
                new EntitiesRosePlugin(),
                new SetupRosePlugin()
        };
        plugins = p;
    }

    /**
     * @throws RosePluginException
     * @see org.softevo.rose.IRosePlugin#reset()
     */
    public void reset() throws RosePluginException {
        for (int i = plugins.length - 1; i >= 0; i--) {
            plugins[i].reset();
        }
    }

    /**
     * @throws RosePluginException
     * @see org.softevo.rose.IRosePlugin#setup()
     */
    public void setup() throws RosePluginException {
        for (int i = 0; i < plugins.length; i++) {
            plugins[i].setup();
        }
    }

    public Job createJob(final IProject project) {
        IProject[] projects = { project };
        return createJob(projects);
    }
    
    public Job createJob(final IProject[] projects) {
        IJobManager jobMan = Platform.getJobManager();
        final IProgressMonitor group = jobMan.createProgressGroup();
        Job job = new Job("Preprocess project") {
            protected IStatus run(IProgressMonitor monitor2) {
                AssistPlugin.getDefault().setPreprocessing(true);
                setProperty(IProgressConstants.ICON_PROPERTY, AssistPlugin.getDefault().getImageRegistry().get("rose"));
                group.beginTask("Preprocess project", 1000);
                boolean jobWasCanceled = false;
                for (int j = 0; j < projects.length && !monitor2.isCanceled() && !jobWasCanceled; j++) {
	                jobWasCanceled = processProject(plugins[0], projects[j], group, ticks[0] / projects.length);
                }
                try {
                    plugins[0].close();
                } catch (RosePluginException e1) {
                    AssistPlugin.logException(e1, Status.WARNING);
                }
	            for (int i = 1; i < plugins.length && !monitor2.isCanceled() && !jobWasCanceled; i++) {
                    IRosePlugin plugin = plugins[i];
                    jobWasCanceled = processProject(plugin, projects[0], group, ticks[i]);
                    try {
                        plugins[i].close();
                    } catch (RosePluginException e1) {
                        AssistPlugin.logException(e1, Status.WARNING);
                    }
                }
                AssistPlugin.getDefault().setPreprocessing(false);
                IStatus result = null;
                Action action = null;
                if (monitor2.isCanceled() || jobWasCanceled) { 
                    result = Status.CANCEL_STATUS;
                    action = getJobCancelledAction();
                } else {
                    result = Status.OK_STATUS;
                    action = getJobCompletedAction();
                }
                if (isModal(this)) {
                    // The progress dialog is still open so
                    // just open the message
                    showResults(action);
                } else {
                    setProperty(IProgressConstants.KEEP_PROPERTY, Boolean.TRUE);
                    setProperty(IProgressConstants.ACTION_PROPERTY, action);
                }
                return result;
            }
            /**
             * @param project
             * @param group
             * @param jobWasCanceled
             * @param i
             * @return
             */
            private boolean processProject(final IRosePlugin plugin, final IProject project, final IProgressMonitor group, int tick) {
                boolean jobWasCanceled = false;
                if (plugin instanceof IRoseCVSPlugin) {
                    try {
                        ((IRoseCVSPlugin) plugin).initCVS(new EclipseRepositoryProvider(project));
                    } catch (RosePluginException e) { //
                    } catch (RepositoryException e) {
                        AssistPlugin.logException(e, Status.WARNING);
                    }
                }
                Job subJob = plugin.createJob(project);
                subJob.setProperty(IProgressConstants.ICON_PROPERTY, AssistPlugin.getDefault().getImageRegistry().get("rose"));
                subJob.setProgressGroup(group, tick);
                //subJob.setUser(true);
                subJob.setPriority(Job.LONG);
                AssistPlugin.debugInfo("Starting " + subJob.getName());
                group.setTaskName(subJob.getName());
                group.subTask(subJob.getName());
                subJob.schedule();
                try {
                    AssistPlugin.debugInfo("Waiting for " + subJob.getName());
                    subJob.join();
                    jobWasCanceled = subJob.getResult().equals(Status.CANCEL_STATUS);
                    AssistPlugin.debugInfo("Finished " + subJob.getName());
                } catch (InterruptedException e) { //empty
                }
                if (plugin instanceof IRoseCVSPlugin) {
                    ((IRoseCVSPlugin) plugin).closeCVS();
                }
                group.setTaskName("Preprocess project");
                return jobWasCanceled;
            }
            public boolean isModal(Job job) {
                Boolean isModal = (Boolean)job.getProperty(IProgressConstants.PROPERTY_IN_DIALOG);
                if(isModal == null) return false;
                return isModal.booleanValue();
             }
        	protected Action getJobCompletedAction() {
        		return new Action("View preprocessing status") {
        			public void run() {
        				MessageDialog.openInformation(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(), 
                          "Preprocessing Complete", 
                          "The preprocessing has been completed.");
        			}
        		};
        	}
        	protected Action getJobCancelledAction() {
        		return new Action("View preprocessing status") {
        			public void run() {
        				MessageDialog.openInformation(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(), 
                          "Preprocessing Cancelled", 
                          "The preprocessing has been cancelled.");
        			}
        		};
        	}
        	protected void showResults(final Action action) {
                Display.getDefault().asyncExec(new Runnable() {
                   public void run() {
                      action.run();
                   }
                });
             }
        };
        job.setProgressGroup(group, 0);
        //j.setSystem(true);
        return job;
    }
    
    
    
    public void run(IProject project) {
        IProject[] projects = { project };
        run(projects);
    }

    public void run(IProject[] projects) {
        Job job = createJob(projects);
        job.setPriority(Job.LONG);
        job.setUser(true);
        job.schedule();
    }
    
    
    public void init(DatabaseProvider db) throws RosePluginException {
        for (int i = 0; i < plugins.length; i++) {
                plugins[i].init(db);
        }
    }

    public void close() throws RosePluginException {
        for (int i = 0; i < plugins.length; i++) {
            plugins[i].close();
        }
    }

    public void initCVS(IRepositoryProvider cvs) throws RosePluginException {
        this.cvs = cvs;
    }
}
