/*
 * $Id: EclipseRepositoryProvider.java,v 1.11.2.2 2005/05/12 10:25:16 zimmerth Exp $
 * 
 * LICENSE:
 */
package org.softevo.rose.ccvs.eclipse;

import java.io.File;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.team.internal.ccvs.core.CVSException;
import org.eclipse.team.internal.ccvs.core.CVSTag;
import org.eclipse.team.internal.ccvs.core.ICVSFolder;
import org.eclipse.team.internal.ccvs.core.ICVSRepositoryLocation;
import org.eclipse.team.internal.ccvs.core.ICVSResource;
import org.eclipse.team.internal.ccvs.core.client.Checkout;
import org.eclipse.team.internal.ccvs.core.client.Command;
import org.eclipse.team.internal.ccvs.core.client.Log;
import org.eclipse.team.internal.ccvs.core.client.Session;
import org.eclipse.team.internal.ccvs.core.client.listeners.ICommandOutputListener;
import org.eclipse.team.internal.ccvs.core.resources.CVSWorkspaceRoot;
import org.eclipse.team.internal.ccvs.core.util.KnownRepositories;
import org.softevo.rose.ccvs.CCVSPlugin;
import org.softevo.rose.ccvs.IRepositoryProvider;
import org.softevo.rose.ccvs.RepositoryException;
import org.softevo.rose.ccvs.listener.ILogOutputListener;
import org.softevo.rose.ccvs.listener.IRepositoryContentListener;
import org.softevo.rose.ccvs.model.LogInfo;
import org.softevo.rose.ccvs.parser.ILogParser;

/**
 * @author Thomas Zimmermann, zimmerth@cs.uni-sb.de
 */
public class EclipseRepositoryProvider implements IRepositoryProvider {

	static final Pattern CVS_RCS = Pattern.compile("RCS file: (.+),v");
	static final Pattern CVS_TERMINATOR = Pattern.compile("={77}");
    
    static final String LINE_SEP = System.getProperty("line.separator");
    private static final String TMP_DIR = System.getProperty("java.io.tmpdir"); 
    
	private Session session;
    private IProject project;
    private ICVSFolder cvsFolder;
    private ICVSRepositoryLocation repository;
    
	/**
	 * 
	 * @param project
	 * @throws CVSException
	 */
	public EclipseRepositoryProvider(IProject project) throws RepositoryException {
		try {
		    this.project = project;
            this.cvsFolder = CVSWorkspaceRoot.getCVSFolderFor(project);
            String root = cvsFolder.getFolderSyncInfo().getRoot();
            this.repository = KnownRepositories.getInstance().getRepository(root);            
            session = new Session(repository, cvsFolder);

            // Setup a linked temp directory
            File actualTmp = new File(new File(TMP_DIR), "rose");
            actualTmp.mkdir();
            IFolder roseTmp = project.getFolder(".rose.tmp");
            if (!roseTmp.exists()) {
                roseTmp.createLink(new Path(actualTmp.getAbsolutePath()), 0, null);
            }

            if (!roseTmp.exists()) {
                roseTmp.create(false /*force*/, true /*local*/, null);
            }

		} catch (CVSException e) {
            throw new RepositoryException(e);
        } catch (CoreException e) {
            CCVSPlugin.logWarning(e.getMessage(), e);
        }
	}
	
	/**
	 * @throws RepositoryException
	 * @see org.softevo.rose.ccvs.IRepositoryProvider#checkout(java.lang.String, java.lang.String)
	 */
	public IFile checkout(String fileName, String revision, IProgressMonitor monitor) throws RepositoryException {
	    
	    try {
	        // Checkout file into contents
	        CVSTag revisionTag = new CVSTag(revision, CVSTag.VERSION);
	        IFolder folder = project.getFolder(".rose.tmp").getFolder(new File(fileName).getParent());
	        IPath targetDirectory = folder.getFullPath().removeFirstSegments(1);
	        myCreateFolder(folder, true, true, null);

	        String targetDirectoryName = targetDirectory.toString();
	        if (targetDirectoryName.startsWith("/")) {
	            targetDirectoryName = targetDirectoryName.substring(1);
	        }

	        //CVSProviderPlugin.getPlugin().setDebugProtocol(false);
	        //CVSProviderPlugin.getPlugin().setDebugging(false);
	        Command.CHECKOUT.execute(session, 
	              new Command.GlobalOption[] {},
	              new Command.LocalOption[] 
	              	{Checkout.DO_NOT_SHORTEN, Checkout.makeDirectoryNameOption(".rose.tmp"), Command.makeTagOption(revisionTag)},
	              new String[] {fileName},
	              new ICommandOutputListener() {
                    public IStatus messageLine(String line,
                            ICVSRepositoryLocation location,
                            ICVSFolder commandRoot, IProgressMonitor monitor) {
                        //System.out.println(line);
                        return ICommandOutputListener.OK;
                    }

                    public IStatus errorLine(String line,
                            ICVSRepositoryLocation location,
                            ICVSFolder commandRoot, IProgressMonitor monitor) {
                        CCVSPlugin.debugInfo(line);
                        return ICommandOutputListener.OK;
                    }
                  }, 
                  monitor);	        

	        return project.getFolder(".rose.tmp").getFile(new Path(fileName));
	        
	    } catch (CVSException e) {
	        throw new RepositoryException("Failed to checkout revision " + revision + " of file " + fileName, e);
	    } catch (CoreException e) {
	        throw new RepositoryException("Failed to checkout revision " + revision + " of file " + fileName, e);
        } 
	}

	/**
     * @param folder
     * @param force
     * @param local
     * @param progress
	 * @throws CoreException
     */
    private void myCreateFolder(IFolder folder, boolean force, boolean local, IProgressMonitor progress) throws CoreException {        
        IContainer parent = folder.getParent();
        if (!parent.exists() && parent instanceof IFolder) {
            myCreateFolder((IFolder) parent, force, local, progress);
        }
        if (!folder.exists()) {
            folder.create(force, local, progress);
        }
    }

    /**
	 * @see org.softevo.rose.ccvs.IRepositoryProvider#getLogsForFile(org.softevo.rose.ccvs.listener.ILogOutputListener, org.softevo.rose.ccvs.parser.ILogParser, java.lang.String)
	 */
	public void getLogsForFile(ILogOutputListener listener, ILogParser parser, String fileName, IProgressMonitor monitor) throws RepositoryException {

	    try {
	         //IStatus state = 
	             Command.LOG.execute(session, 
                 new Command.GlobalOption[] {}, 
                 new Command.LocalOption[] {}, 
                 new String[] {fileName}, 
	             new DefaultLogOutputListener(listener, parser),
                 monitor);
	          
        } catch (CVSException e) {
	        throw new RepositoryException("Failed to get log for file " + fileName, e);
        }

	}

	/**
	 * @see org.softevo.rose.ccvs.IRepositoryProvider#getLogs(org.softevo.rose.ccvs.listener.ILogOutputListener, org.softevo.rose.ccvs.parser.ILogParser)
	 */
	public void getLogs(final ILogOutputListener listener, final ILogParser parser, IProgressMonitor monitor)  throws RepositoryException {
	    
	    try {
            //IStatus state = 
                Command.LOG.execute(session,
                new Command.GlobalOption[] {},
                new Command.LocalOption[] {},
                new ICVSResource[] {cvsFolder},
                new DefaultLogOutputListener(listener, parser),
                monitor);
                
        } catch (CVSException e) {
            throw new RepositoryException("Failed to get all logs", e);
        }

	}

	/**
     * @see org.softevo.rose.ccvs.IRepositoryProvider#getRepositoryContent(org.softevo.rose.ccvs.listener.IRepositoryContentListener)
     */
	public void getRepositoryContent(final IRepositoryContentListener listener, IProgressMonitor monitor) throws RepositoryException {
	    try {
	         // IStatus state =
            	Command.LOG.execute(session, 
            	    new Command.GlobalOption[] {},
            	    new Command.LocalOption[] {Log.RCS_FILE_NAMES_ONLY}, 
            	    new ICVSResource[] {cvsFolder},
            	    new ICommandOutputListener() {
            	    	// assert !cvsFolder.getFolderSyncInfo().getRepository().equals("");
            	    	final String MODULE = File.separator + cvsFolder.getFolderSyncInfo().getRepository() + File.separator;
            	        final String ROOT = repository.getRootDirectory(); 
            	    	final int POS = ROOT.length(); 
                        public IStatus messageLine(String line,
                                ICVSRepositoryLocation location,
                                ICVSFolder commandRoot, IProgressMonitor monitor) {
                            if (line.startsWith(ROOT)) {
                                String fileName = line.substring(POS);
                                if (fileName.indexOf("/Attic/")<0) {
                                    listener.doFile(fileName);
                                } else {
                                    listener.doDeletedFile(fileName.replaceFirst("/Attic/", "/"));                                    
                                }
                            }
                            return ICommandOutputListener.OK;
                        }

                        public IStatus errorLine(String line,
                                ICVSRepositoryLocation location,
                                ICVSFolder commandRoot, IProgressMonitor monitor) {
                            if (line.startsWith("cvs log: Logging ")) {
                                String directory = line.substring(17);  // "cvs log: Logging".length()
                                directory = (directory.equals(".") ? "" : directory + File.separator);
                                listener.doDirectory(MODULE + directory);
                            }
                            return ICommandOutputListener.OK;
                        }
                    },
                    monitor);
	          
       } catch (CVSException e) {
	        throw new RepositoryException("Failed to get repository content", e);
       }
	}
	
	public void unshareResource(IResource resource, IProgressMonitor progress) throws RepositoryException {
	    ICVSResource cvsResource = CVSWorkspaceRoot.getCVSResourceFor(resource);
	    try {
            cvsResource.unmanage(progress);
        } catch (CVSException e) {
	        throw new RepositoryException("Failed to unshare resource", e);
        }
	}

	/**
	 * @see org.softevo.rose.ccvs.IRepositoryProvider#open()
	 */
	public void open() throws RepositoryException {
        try {
			session.open(new NullProgressMonitor());
		} catch (CVSException e) {
		    throw new RepositoryException(e);
        }
	}

	/**
	 * @see org.softevo.rose.ccvs.IRepositoryProvider#close()
	 */
	public void close() {
		session.close();
        IFolder roseTmp = project.getFolder(".rose.tmp");
        try {
            unshareResource(roseTmp, null);
        } catch (RepositoryException e) {
            CCVSPlugin.logException(e, IStatus.WARNING);
        }
        try {
            roseTmp.delete(true /*force*/, false /*no history*/, null /*no progress*/);
        } catch (CoreException e) {
            CCVSPlugin.logException(e, IStatus.WARNING);
        }
        File actualTmp = new File(new File(TMP_DIR), "rose");
        deleteAllDirsAndFiles(actualTmp);
	}

    // Process all files and directories under dir
    public static void deleteAllDirsAndFiles(File dir) {
        if (dir.isDirectory()) {
            String[] children = dir.list();
            for (int i=0; i<children.length; i++) {
                deleteAllDirsAndFiles(new File(dir, children[i]));
            }
        }
        dir.delete();
    }
	
	/**
	 * Hack the constructor of the Command.LocalOption class visible.
	 * 
	 * @author Thomas Zimmermann, zimmerth@cs.uni-sb.de
	 */
	public static class RoseOption extends Command.LocalOption {

        /**
         * @param option
         */
        public RoseOption(String option) {
            super(option);
        }
        
        /** 
         * @param option
         * @param argument
         */
        public RoseOption(String option, String argument) {
            super(option, argument);
        }	    
	}
	
	/**
	 * Parse the log output and call log listeners.
	 * 
	 * @author Thomas Zimmermann, zimmerth@cs.uni-sb.de
	 */
	private class DefaultLogOutputListener implements ICommandOutputListener {

	    /* the position where the filename starts */
	    final int POS = repository.getRootDirectory().length(); 
	    
	    /* the filename */
  	  	String fileName = null;
  	  	
  	  	/* the log output for this file */
	    StringBuffer content = null;

        private ILogOutputListener listener = null;

        private ILogParser parser = null; 

	    public DefaultLogOutputListener(ILogOutputListener listener, ILogParser parser) {
	        this.listener = listener;
	        this.parser = parser;
	    }
	    
  	  	public IStatus messageLine(String line,
              ICVSRepositoryLocation location,
              ICVSFolder commandRoot, IProgressMonitor monitor) {
  	  	    
  	  	    Matcher mRCS = CVS_RCS.matcher(line);
  	  	    Matcher mTerminator = CVS_TERMINATOR.matcher(line);
  	  	    
  	  	    if (mRCS.matches()) {
  	  	        // Create new buffer and filename
  	  	        content = new StringBuffer(4000);
  	  	        content.append(line);
  	  	        content.append(LINE_SEP);
  	  	        fileName = mRCS.group(1).substring(POS).replaceFirst("/Attic/","/");
  	  	    } else if (mTerminator.matches()) {
  	  	        content.append(line);
  	  	        content.append(LINE_SEP);
  	  	        // Parse the output
  	  	        LogInfo logInfo = parser.getLogInfo(fileName, content.toString());
  	          
  	  	        // Call the listener
  	  	        listener.doLogInfo(logInfo);
  	  	    } else if (content != null){  	  	        
  	  	        content.append(line);
  	  	        content.append(LINE_SEP);
  	  	    }
  	  	    return ICommandOutputListener.OK;
  	  	}

  	  	public IStatus errorLine(String line,
              ICVSRepositoryLocation location,
              ICVSFolder commandRoot, IProgressMonitor monitor) {
  	  	    return ICommandOutputListener.OK;
  	  	}	    
	}

}
