/*
 * $Id: ModificationItem.java,v 1.7.2.1 2005/04/09 12:44:05 zimmerth Exp $
 * 
 * LICENSE:
 */
package org.softevo.rose.compare;

import java.io.IOException;
import java.io.InputStream;
import java.util.Arrays;
import java.util.LinkedList;

import org.eclipse.compare.ITypedElement;
import org.eclipse.compare.ResourceNode;
import org.eclipse.compare.structuremergeviewer.DocumentRangeNode;

/**
 * @author Thomas Zimmermann, zimmerth@cs.uni-sb.de
 */
public class ModificationItem {

    /** Constant for an added item. */
    public static final int ADDED = 1;

    /** Constant for a changed item. */
    public static final int CHANGED = 2;

    /** Constant for a deleted item. */
    public static final int DELETED = 3;

    public static final String[] TYPE_DESCRIPTIONS = {
        null, "ADDED", "CHANGED", "DELETED"
    };
    
    private String attachedFileName = null;
    private Object node = null;
    private int type = -1;
    
    public ModificationItem(DocumentRangeNode node, int type) {
        if (node == null) {
            throw new IllegalArgumentException("node must not be null");
        }
        this.node = node;
        this.type = type;
    }

    public ModificationItem(ResourceNode node, int type) {
        if (node == null) {
            throw new IllegalArgumentException("node must not be null");
        }
        this.node = node;
        this.type = type;
    }
    
    public ModificationItem(ITypedElement node, int type) {
        if (node == null) {
            throw new IllegalArgumentException("node must not be null");
        }
        this.node = node;
        this.type = type;
    }
   
    

    public static ModificationItem createModificationItem(Object obj, int type) {
        if (obj instanceof DocumentRangeNode) {
            return new ModificationItem((DocumentRangeNode) obj, type);
        } else if (obj instanceof DocumentRangeNode) {
            return new ModificationItem((ResourceNode) obj, type);
        } else if (obj instanceof ITypedElement) {
            return new ModificationItem((ITypedElement) obj, type);
        }
        return null;
    }

    
    public static LinkedList createModificationItems(Object obj, int type) {
        LinkedList result = new LinkedList();
        if (obj instanceof DocumentRangeNode) { // thus obj is not null
            DocumentRangeNode drn = (DocumentRangeNode) obj;
            result.add(new ModificationItem(drn, type));
            
            if (type == ADDED || type == DELETED) {
                LinkedList stack = new LinkedList();
                if (drn.getChildren() != null) stack.addAll(Arrays.asList(drn.getChildren()));
                while (!stack.isEmpty()) {
                    Object o = stack.removeFirst();
                    if (o instanceof DocumentRangeNode) { // thus o is not null
                        DocumentRangeNode d = (DocumentRangeNode) o;
                        result.add(new ModificationItem(d, type));
                        if (d.getChildren() != null) stack.addAll(Arrays.asList(d.getChildren()));                        
                    } else if (o instanceof ResourceNode) {
                        result.add(new ModificationItem((ResourceNode) o, type));
                    } else if (o instanceof ITypedElement) {
                        result.add(new ModificationItem((ITypedElement) o, type));
                    }
                }
            }
        } else if (obj instanceof DocumentRangeNode) {
            result.add(new ModificationItem((ResourceNode) obj, type));
        } else if (obj instanceof ITypedElement) {
            result.add(new ModificationItem((ITypedElement) obj, type));
        }
        return result;
    }

    
    
    public boolean isDocumentRangeNode() {
        return node instanceof DocumentRangeNode;
    }

    public boolean isResourceNode() {
        return node instanceof ResourceNode;
    }

    public boolean isITypedElement() {
        return node instanceof ITypedElement;
    }

    // getters
    public DocumentRangeNode getDocumentRangeNode() {
        return isDocumentRangeNode() ? (DocumentRangeNode) node : null;
    }
    public ResourceNode getResourceNode() {
        return isResourceNode() ? (ResourceNode) node : null;
    }
    public ITypedElement getITypedElement() {
        return isITypedElement() ? (ITypedElement) node : null;
    }
    public int getType() {
        return type;
    }
    public void changeType(int i) {
        type = i;
    }
    public String getTypeDescription() {
        return TYPE_DESCRIPTIONS[type];
    }
    public String getID() {
        if (node instanceof DocumentRangeNode)
            return ((DocumentRangeNode) node).getId();
        if (node instanceof ResourceNode)
            return ((ResourceNode) node).getName();
        if (node instanceof ITypedElement)
            return ((ITypedElement) node).getName();
        return null;
    }
    public String getContent() {
        if (type==DELETED) return null;
        if (node instanceof DocumentRangeNode) {
            DocumentRangeNode drn = (DocumentRangeNode) node;
            try {
                return ""; // FIXME drn.getDocument().get(drn.getRange().getOffset(),drn.getRange().getLength());
            } catch (Exception e) { //empty
            }
        }
        return null;
    }
    protected String readInputStream(InputStream is) {
        StringBuffer result = new StringBuffer(1000);
        int len = 0;
        byte[] bytes = new byte[1000];
        try {
            while ((len=is.read(bytes)) >= 0) {
                result.append(new String(bytes, 0, len));
            }
        } catch (IOException e) {
            return null;
        }
        return result.toString();
    }
    
    public boolean equals(Object obj) {
        if (obj instanceof ModificationItem && getID() != null) {
            ModificationItem mi = (ModificationItem) obj;
            if (attachedFileName != null) {
                return getID().equals(mi.getID()) && attachedFileName.equals(mi.getAttachedFileName());
            }
            return getID().equals(mi.getID()) && mi.getAttachedFileName() == null;
        }
        return false;
    }
    public String getAttachedFileName() {
        return attachedFileName;
    }
    public void setAttachedFileName(String s) {
        attachedFileName = s;
    }
}
