/*
 * $Id: DatabaseOperations.java,v 1.13 2005/03/01 12:55:25 zimmerth Exp $
 * 
 * LICENSE:
 */
package org.softevo.rose.data.mirror.db;

import java.io.File;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.softevo.rose.ccvs.model.BranchItem;
import org.softevo.rose.ccvs.model.LogInfo;
import org.softevo.rose.ccvs.model.RevisionItem;
import org.softevo.rose.ccvs.model.TagItem;
import org.softevo.rose.ccvs.model.TransactionItem;
import org.softevo.rose.data.mirror.MirrorPlugin;
import org.softevo.rose.db.DBConnectionProperties;
import org.softevo.rose.db.DatabaseProvider;
import org.softevo.rose.db.jdbc.DatabaseProperties;
import org.softevo.util.StringUtils;


/**
 * @author Thomas Zimmermann, zimmerth@cs.uni-sb.de
 */
public class DatabaseOperations {

        
    private Connection connection = null;
    private DBConnectionProperties properties;
    
    private final String INT;
    private final String TEXT;
    private final String VARCHAR;
    private final String TIMESTAMP;
    
    private Statement stmt;
    private DatabaseProvider provider;
    private PreparedStatement stmtBranch;
    private PreparedStatement stmtDirectory;
    private PreparedStatement stmtFile;
    private PreparedStatement stmtRevision;
    private PreparedStatement stmtTag;
    private PreparedStatement stmtTransaction;
    private PreparedStatement stmtUpdateTxInRevision;
    
    public DatabaseOperations(DatabaseProvider provider) {
        this.provider = provider;

        properties = provider.getConnectionProperties();
        INT = properties.getDatabaseProperties().getTypeMapping(DatabaseProperties.TP_INT);
        TEXT = properties.getDatabaseProperties().getTypeMapping(DatabaseProperties.TP_TEXT);
        VARCHAR = properties.getDatabaseProperties().getTypeMapping(DatabaseProperties.TP_VARCHAR);
        TIMESTAMP = properties.getDatabaseProperties().getTypeMapping(DatabaseProperties.TP_TIMESTAMP);

    }
    

    public void init() throws SQLException {
        connection = provider.getConnection();
        stmt = connection.createStatement();  
    }
    
    public void setupStmts() throws SQLException {
        stmtBranch = createInsertStatement(getTableName(Tables.TABLE_BRANCHES), 5);
        stmtDirectory = createInsertStatement(getTableName(Tables.TABLE_DIRECTORIES), 2);
        stmtFile = createInsertStatement(getTableName(Tables.TABLE_FILES), 8);
        stmtRevision = createInsertStatement(getTableName(Tables.TABLE_REVISIONS), 9);
        stmtTag = createInsertStatement(getTableName(Tables.TABLE_TAGS), 3);
        stmtTransaction = createInsertStatement(getTableName(Tables.TABLE_TRANSACTIONS), 7);
        stmtUpdateTxInRevision = connection.prepareStatement
        	("UPDATE " + getTableName(Tables.TABLE_REVISIONS) + " SET TransaktionID=? WHERE FileID=? AND RevisionID=?;");
    }
 
    
    public void close() throws SQLException {
        stmtBranch.close();
        stmtDirectory.close();
        stmtFile.close();
        stmtRevision.close();
        stmtTag.close();
        stmtTransaction.close();
        stmtUpdateTxInRevision.close();
        stmt.close();
    }
    
    
    public void addConstraintUnique
    	(String constraintName, String tableName, String[] columnList) 
    	throws SQLException {
        
		String query = "ALTER TABLE " + tableName + " "
			+ "ADD CONSTRAINT " + constraintName + " "
			+ "UNIQUE (" + StringUtils.join(columnList, ",") + ");";
		executeUpdate(query);		
    }
    
    
    public void addConstraintForeignKey
    	(String constraintName, String tableName, String[] columnList, 
    	 String targetTable, String[] targetColumnList) throws SQLException {
        
		String query = "ALTER TABLE " + tableName + " "
			+ "ADD CONSTRAINT " + constraintName + " "
			+ "FOREIGN KEY (" + StringUtils.join(columnList, ",") + ") "
			+ "REFERENCES " + targetTable 
				+ " (" + StringUtils.join(targetColumnList, ",") + ") "
				+ "ON UPDATE CASCADE";
		executeUpdate(query);	
    }
    
    
	public void createTableDirectorys(String tableName) throws SQLException {
	    tableName = tableName == null ? getTableName(Tables.TABLE_DIRECTORIES) : tableName;
		String query = "CREATE TABLE " + tableName + " ( "
			+ "DirectoryName " + VARCHAR + "(250) NOT NULL, "
			+ "Depth " + INT + " NOT NULL, "
			+ "CONSTRAINT " + tableName + "_pkey "
				+ "PRIMARY KEY (DirectoryName) "
			+ ");";
		executeUpdate(query);
	}
	
	
	public void createTableFiles(String tableName) throws SQLException {
	    tableName = tableName == null ? getTableName(Tables.TABLE_FILES) : tableName;
	    String query = "CREATE TABLE " + tableName + " ( "
			+ "FileID " + INT + " NOT NULL, "
			+ "FileName " + VARCHAR + "(100) NOT NULL, "
			+ "DirectoryName " + VARCHAR + "(250) NOT NULL, "
			+ "QualifiedFileName " + VARCHAR + "(250) NOT NULL, "
			+ "FileExtension " + VARCHAR + "(20), "
			+ "Depth " + INT + " NOT NULL, "
			+ "KeywordExpansion " + VARCHAR + "(10), "
			+ "NumberOfRevisions " + INT + ", "
			+ "CONSTRAINT " + tableName + "_fkey "
				+ "FOREIGN KEY (DirectoryName) REFERENCES "
				+ getTableName(Tables.TABLE_DIRECTORIES) + " (DirectoryName), "
			+ "CONSTRAINT " + tableName + "_pkey "
				+ "PRIMARY KEY (FileID) "
			+ ");";
		executeUpdate(query);
	}
	

	public void createTableTransactions(String tableName) throws SQLException {	
	    tableName = tableName == null ? getTableName(Tables.TABLE_TRANSACTIONS) : tableName;
		String query = "CREATE TABLE " + tableName + " ( "
			+ "TransaktionID " + INT + " NOT NULL, "
			+ "Author " + VARCHAR + "(60) NOT NULL, "
			+ "Message " + TEXT + " NOT NULL, "
			+ "MessageMD5 " + VARCHAR + "(32) NOT NULL, "
			+ "BeginTime " + TIMESTAMP + " NOT NULL, "
			+ "EndTime " + TIMESTAMP + " NOT NULL, "
			+ "IsNoise " + VARCHAR + "(1) NOT NULL, "
			+ "CONSTRAINT " + tableName + "_pkey "
				+ "PRIMARY KEY (TransaktionID) "
			+ ");";
		executeUpdate(query);
	}


	public void createTableRevisions(String tableName) throws SQLException {
	    tableName = tableName == null ? getTableName(Tables.TABLE_REVISIONS) : tableName;
		String query = "CREATE TABLE " + tableName + " ( "
			+ "FileID " + INT + " NOT NULL, "
			+ "RevisionID " + VARCHAR + "(50) NOT NULL, "
			+ "TransaktionID " + INT + " NOT NULL, "
			+ "CheckinTime " + TIMESTAMP + " NOT NULL, "
			+ "Plus " + INT + ", "
			+ "Minus " + INT + ", "
			+ "State " + VARCHAR + "(20), "
			+ "BranchPrefix " + VARCHAR + "(50), "
			+ "LogPosition " + INT + " NOT NULL, "
			+ "CONSTRAINT " + tableName + "_f_fkey "
				+ "FOREIGN KEY (FileID) REFERENCES " 
				+ getTableName(Tables.TABLE_FILES) + " (FileID) "
				+ "ON UPDATE CASCADE, "
			+ "CONSTRAINT " + tableName + "_t_fkey "
				+ "FOREIGN KEY (TransaktionID) REFERENCES " 
				+ getTableName(Tables.TABLE_TRANSACTIONS) + " (TransaktionID) "
				+ "ON UPDATE CASCADE, "
			+ "CONSTRAINT " + tableName + "_pkey "
				+ "PRIMARY KEY (FileID, RevisionID) "
			+ ");";
		executeUpdate(query);
	}


	public void createTableBranches(String tableName) throws SQLException {	
	    tableName = tableName == null ? getTableName(Tables.TABLE_BRANCHES) : tableName;
		String query = "CREATE TABLE " + tableName + " ( "
			+ "FileID " + INT + " NOT NULL, "
			+ "BranchPrefix " + VARCHAR + "(50) NOT NULL, "
			+ "OriginRevision " + VARCHAR + "(50) NOT NULL, "
			+ "BranchName " + VARCHAR + "(100) NOT NULL, "
			+ "InternalRevision " + VARCHAR + "(50) NOT NULL, "
			+ "CONSTRAINT " + tableName + "_fkey "
				+ "FOREIGN KEY (FileID, OriginRevision) REFERENCES "
				+ getTableName(Tables.TABLE_REVISIONS) + " (FileID, RevisionID) "
				+ "ON UPDATE CASCADE, "
			+ "CONSTRAINT " + tableName + "_pkey "
				+ "PRIMARY KEY (FileID, BranchPrefix) "
			+ ");";
		executeUpdate(query);
	}
	

	public void createTableTags(String tableName) throws SQLException {		
	    tableName = tableName == null ? getTableName(Tables.TABLE_TAGS) : tableName;
		String query = "CREATE TABLE " + tableName + " ( "
			+ "FileID " + INT + " NOT NULL, "
			+ "TagName " + VARCHAR + "(100) NOT NULL, "
			+ "RevisionID " + VARCHAR + "(50) NOT NULL, "
			+ "CONSTRAINT " + tableName + "_fkey "
				+ "FOREIGN KEY (FileID) REFERENCES " 
				+ getTableName(Tables.TABLE_FILES) + " (FileID) "
				+ "ON UPDATE CASCADE, "
			+ "CONSTRAINT " + tableName + "_pkey "
				+ "PRIMARY KEY (FileID, TagName) "
			+ ");";
		executeUpdate(query);
	}

	
	/**
	 * 
	 * @param tableName
	 * @throws SQLException
	 */
	public void dropTable(String tableName) throws SQLException {		
		String query = "DROP TABLE " + tableName + ";";
	    executeUpdate(query);
	}

	
	/**
	 * 
	 * @param indexName
	 * @throws SQLException
	 */
	public void dropIndex(String indexName) throws SQLException {		
		String query = "DROP INDEX " + indexName + ";";
	    executeUpdate(query);
	}

	
	public void dropConstraint(String tableName, String constraintName) 
		throws SQLException {
	    
		String query = "ALTER TABLE " + tableName + " "
			+ "DROP CONSTRAINT " + constraintName; // + "RESTRICT";
		executeUpdate(query);
	}
	
	/**
	 * Tests whether a table exists. 
	 * @param tableName
	 * @return
	 */
	public boolean existsTable(String tableName) {
	    // Use "SELECT COUNT(*) FROM " + tableName + ";";
	    // But this should be faster (except for braindead databases):
	    String query = "SELECT 1 FROM " + tableName + " WHERE 1=0;";
	    try {
            stmt.executeQuery(query);
        } catch (SQLException e) {
            return false;
        }
	    return true;
	}
	

	public String getTableName(String tableID) {
	    return Tables.PREFIX_CVS + tableID;
	}

	
	/**
	 * Inserts a branch tuple into Table Branches
	 * 
	 * @param fileID  The fileID for which a branch begins
	 * @param branch  The revision of the first branch checkin.
	 * @param parentRevision  The revision where the branch begins
	 * @throws SQLException
	 */
	public void insertBranch(int fileID, BranchItem branch) 
		throws SQLException {
	
	    stmtBranch.setInt(1, fileID);
	    stmtBranch.setString(2, branch.getPrefixRevision());
	    stmtBranch.setString(3, branch.getOriginRevision());
	    stmtBranch.setString(4, branch.getBranchName());
	    stmtBranch.setString(5, branch.getInternalRevision());
	    stmtBranch.execute();	    
	}

	
	/**
	 * Inserts a directory tuple into table Directorys.
	 * The depth of the directory is computed by simply counting the 
	 * occurences of "/" in directory.
	 * <P>
	 * Examples: mozilla/ has depth 0, mozilla/build has depth 1, etc.
	 * 
	 * @param directory  The directory name.
	 * @throws SQLException
	 */
	public void insertDirectory(String directory) throws SQLException {
	    directory = directory.replace(File.separatorChar, '/');
		int depth = StringUtils.countFrequency(directory, "/") - 1 ;
		stmtDirectory.setString(1, directory);
		stmtDirectory.setInt(2, depth);
		stmtDirectory.execute();
	}


	/**
	 * Inserts a file tuple into Table Files. All attributes like directory,
	 * filename, file extension or depth are computed in this method.
	 * 
	 * @param fileID  The unique identifier of the file.
	 * @param logInfo
	 * @throws SQLException
	 */
	public void insertFile(int fileID, LogInfo logInfo) 
		throws SQLException {

	    String fileName = logInfo.getFileName();
	    
		// Determine directory, file name and extension
		String shortFileName = logInfo.getFileShortName();
		String directoryName = logInfo.getDirectoryName();
		if (shortFileName != null && directoryName != null) {
			String extension = logInfo.getFileExtension();
			int depth = logInfo.getFileDepth();

			// Ignore file extension of length more than 20
			if (extension != null && extension.length() > 20) {
				extension = null;
			}

			// Update database
			stmtFile.setInt(1, fileID);
			stmtFile.setString(2, shortFileName);
			stmtFile.setString(3, directoryName);
			stmtFile.setString(4, fileName);
			stmtFile.setString(5, extension);
			stmtFile.setInt(6, depth);
			stmtFile.setString(7, logInfo.getKeywordExpansion());
			stmtFile.setInt(8, logInfo.getSelectedRevisions());
			stmtFile.execute();

		} else {
			MirrorPlugin.logWarning("Could not match file: [" + fileName + "]", null);
		}

	}


	/**
	 * Inserts a revision tuple into table Revisions. A revision is identified
	 * by a fileID and the revision label, e.g "1.1"
	 * 
	 * @param fileID  The file identifier
	 * @param revision  The revision label, e.g. "1.1"
	 * @param transactionID  The transaktion identifier.
	 * @param commitDate  The date when the revision was commited.
	 * @param plusLines  The number of added lines.
	 * @param minusLines  The number of removed lines.
	 * @param status  
	 *        The CVS status of the revision (<TT>Exp</TT> or <TT>dead</TT>)
	 * @param lineitem  The position in the cvs log message 
	 * 		  (1 is newer 2 if on same branch)
	 * @throws SQLException
	 */
	public void insertRevision(int transactionID, int fileID, RevisionItem revision) 
		throws SQLException {

		// Insert revision into table Revision
		stmtRevision.setInt(1, fileID);
		stmtRevision.setString(2, revision.getRevision());
		stmtRevision.setInt(3, transactionID);
		stmtRevision.setTimestamp(4, revision.getTimestamp());
		stmtRevision.setInt(5, revision.getPlusLines());
		stmtRevision.setInt(6, revision.getMinusLines());
		stmtRevision.setString(7, revision.getState());
		stmtRevision.setString(8, revision.getBranchPrefix());
		stmtRevision.setInt(9, revision.getPosition());
		stmtRevision.execute();
	}


	/**
	 * Inserts a tag into table Tags. The tag marks the revision identified
	 * by (fileID, revision)
	 * 
	 * @param fileID  The file identifier.
	 * @param tag  The tag name.
	 * @param revision  The revision label.
	 * @throws SQLException
	 */
	public void insertTag(int fileID, TagItem tag) throws SQLException {
	    stmtTag.setInt(1, fileID);
	    stmtTag.setString(2, tag.getTagName());
	    stmtTag.setString(3, tag.getRevision());
	    stmtTag.execute();
	}


	/**
	 * Inserts a transaktion tuple into Transaktions.
	 * 
	 * @param transactionID  The unique transaktion identifier.
	 * @param autor  The commiter/autor of the transaktion
	 * @param message  The log message for the transaktion
	 * @param beginDate  The begin of the transaktion
	 * @param endDate  The end of the transaktion
	 * @throws SQLException
	 */
	public void insertTransaktion(int transactionID, RevisionItem revision) 
		throws SQLException {

	    stmtTransaction.setInt(1, transactionID);
	    stmtTransaction.setString(2, revision.getAuthor());
	    stmtTransaction.setString(3, revision.getMessage());
	    stmtTransaction.setString(4, revision.getMessageMD5());
	    stmtTransaction.setTimestamp(5, revision.getTimestamp());
	    stmtTransaction.setTimestamp(6, revision.getTimestamp());
	    stmtTransaction.setString(7, "N"); // no noise
	    stmtTransaction.execute();
	}

	
	/**
	 * Inserts a transaktion tuple into Transaktions.
	 * 
	 * @param transactionID  The unique transaktion identifier.
	 * @param autor  The commiter/autor of the transaktion
	 * @param message  The log message for the transaktion
	 * @param beginDate  The begin of the transaktion
	 * @param endDate  The end of the transaktion
	 * @throws SQLException
	 */
	public void insertTransaktion(int transactionID, TransactionItem tx) 
		throws SQLException {

	    stmtTransaction.setInt(1, transactionID);
	    stmtTransaction.setString(2, tx.getAuthor());
	    stmtTransaction.setString(3, tx.getMessage());
	    stmtTransaction.setString(4, tx.getMessageMD5());
	    stmtTransaction.setTimestamp(5, tx.getBeginTime());
	    stmtTransaction.setTimestamp(6, tx.getEndTime());
	    stmtTransaction.setString(7, tx.getNoise()); // no noise
	    stmtTransaction.execute();
	}

	
	public void renameTable(String oldName, String newName) 
		throws SQLException {
	    
		String query = "ALTER TABLE " + oldName	+ " RENAME TO " + newName;
		executeUpdate(query);
	}
	
	
	public void setAutoCommit(boolean autoCommit) throws SQLException {
	    connection.setAutoCommit(autoCommit);
	}
	
	
	public void updateTxInRevision(int fileID, String revisionID, int tID)
		throws SQLException {
	
		stmtUpdateTxInRevision.setInt(1, tID);
		stmtUpdateTxInRevision.setInt(2, fileID);
		stmtUpdateTxInRevision.setString(3, revisionID);
		stmtUpdateTxInRevision.execute();
	}
	
	private PreparedStatement createInsertStatement(String tableName, int numberOfFields) throws SQLException {
	    StringBuffer query = new StringBuffer("INSERT INTO " + tableName + " VALUES (");
	    for (int i = 0; i < numberOfFields; i++) {
            if (i>0) {
                query.append(",?");
            } else {
                query.append("?");
            }
        }
	    query.append(");");
	    PreparedStatement result = connection.prepareStatement(query.toString());
		return result;
	}	

	
	public void executeUpdate(String query) throws SQLException {
	    MirrorPlugin.debugInfo(query);
		stmt.executeUpdate(query);	    
	}

	public ResultSet executeQuery(String query) throws SQLException {
	    return stmt.executeQuery(query);
	}

}
