/*
 * $Id: DBConnectionEditor.java,v 1.9 2005/01/12 16:06:13 dallmeier Exp $
 * 
 * LICENSE:
 */
package org.softevo.rose.db.preferences;

import java.util.Vector;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IMessageProvider;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.wizard.IWizard;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.FontMetrics;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.softevo.rose.db.DBConnectionProperties;
import org.softevo.rose.db.DbPlugin;
import org.softevo.rose.db.jdbc.DatabaseProperties;
import org.softevo.rose.db.jdbc.JdbcPlugin;

/** This class contains the visual elements needed
 *  to edit connection properties. <BR>
 *  This class supports an edit and a new mode.
 *  
 */
public class DBConnectionEditor {
  
  /** Integer constant for edit existing mode. <BR>
   */
  public static final int MODE_EDIT = 1;
  
  /** Integer constant for new connectionmode. <BR>
   */
  public static final int MODE_NEW = 2;
  
  /** The parent of the editor. <BR>
   */
  private Composite parent = null;
  
  /** The root composite. <BR>
   */
  private Composite rootPanel = null;
  
  /** Editor for name. <BR>
   */
  private StringFieldEditor nameField;
  
  /** Editor for url. <BR>
   */
  private StringFieldEditor urlField;
  
  /** Editor for user name. <BR>
   */
  private StringFieldEditor userNameField;
  
  /** Editor for password. <BR>
   */
  private StringFieldEditor passwordField;
  
  /** Combo with database drivers. <BR>
   */
  private Combo driverCombo = null;
  
  /** The list of drivers. <BR>
   */
  private Vector drivers = null;
  
  /** The connection properties. <BR>
   */
  private DBConnectionProperties properties = null;
  
  /** The mode of the editor. <BR>
   */
  private int mode = MODE_EDIT;

  private IWizard wizard = null;

  private WizardPage wizardPage; 
  
  /** Creates a new editor. <BR>
   *  @param parent the visual parent
   *  @param properties the properties to edit (<code>null</code> if new)
   *  @param mode the editor mode (see constants)
   */
  public DBConnectionEditor(Composite parent, 
                            DBConnectionProperties properties,
                            int mode) {
    this.parent = parent;
    this.properties = properties;
    this.mode = mode;
    createElements();
  }
  
  /** Creates the elements of the editor. <BR>
   */
  private void createElements() {
    rootPanel = createRootPanel();
    //if(inNewMode()) {
    //  createWarnLabel(rootPanel);
    //}
    createEditorFields(rootPanel);
    if (inEditMode()) {
      initFields(properties);
    }
    applyToStatusLine(validateSettings());
    
    //if (inNewMode()) {
    //  changeWarnLabel();
    //}
  }
  
  public void refresh() {
      applyToStatusLine(validateSettings());
  }
  
  
  /** Sets initial values for the fields. <BR>
   *  @param properties the properties object with initial values
   */
  private void initFields(DBConnectionProperties properties) {
    nameField.setStringValue(properties.getName());
    urlField.setStringValue(properties.getDatabaseURL());
    userNameField.setStringValue(properties.getUserName());
    passwordField.setStringValue(properties.getPassword());
    driverCombo.setText(properties.getDatabaseProperties().getName());
  }
  
  /** Creates the fields for the editor. <BR>
   */
  private void createEditorFields(Composite parent) {
    Composite container;
    GridLayout layout;
    GridData gridData;
    
    IPropertyChangeListener ipcl = new IPropertyChangeListener() {
        public void propertyChange(PropertyChangeEvent event) {
            applyToStatusLine(validateSettings());
        }
      };
    
    container = new Composite(parent, SWT.NULL);
    layout = new GridLayout();
    layout.numColumns = 1;
    gridData = new GridData(GridData.FILL_BOTH);
    //gridData.grabExcessHorizontalSpace = true;
    //gridData.grabExcessVerticalSpace = true;
    container.setLayout(layout);
    container.setLayoutData(gridData);

    Group g = createGroup(container, "Location");
    Composite g1 = createGroupContainer(g);
    nameField = new StringFieldEditor("namefield",
                                      "&Name:",
                                      g1);
    if (inNewMode()) {
      nameField.setPropertyChangeListener(ipcl);
    } else if (inEditMode()) {
      nameField.getTextControl(g1).setEditable(false);
    }
    urlField = new StringFieldEditor("jdbcurl",
                                     "&Database path:",
                                     g1);
    urlField.setPropertyChangeListener(ipcl);

    g = createGroup(container, "Authentication");
    g1 = createGroupContainer(g);
    userNameField = new StringFieldEditor("usernamefield",
                                          "&User:",
                                          g1);
    userNameField.setPropertyChangeListener(ipcl);
    passwordField = new StringFieldEditor("passwordfield",
                                          "&Password:",
                                          g1);
    passwordField.getTextControl(g1).setEchoChar('*');
    passwordField.setPropertyChangeListener(ipcl);
    createWarning(g, "Passwords are stored on your computer in a file that is not encryted and easy to read for an intruder.");

    g = createGroup(container, "Connection");
    createDriverCombo(g);

  }
  
  protected Group createGroup(Composite parent, String text) {
      Group group = new Group(parent, SWT.NULL);
      group.setText(text);
      GridData data = new GridData(GridData.FILL_HORIZONTAL);
      data.horizontalSpan = 2;
      
      group.setLayoutData(data);
      GridLayout layout = new GridLayout();
      layout.numColumns = 2;
      layout.marginHeight = 5;
      layout.marginWidth = 5;
      group.setLayout(layout);
      return group;
  }
  
  protected void createWarning(Composite composite, String message) {
      Composite warningComposite = new Composite(composite, SWT.NONE);
      GridLayout layout = new GridLayout();
      layout.numColumns = 2;
      layout.marginHeight = 0;
      //layout.marginWidth = 0;
      warningComposite.setLayout(layout);
      GridData data = new GridData(GridData.FILL_HORIZONTAL);
      data.horizontalSpan = 3;
      warningComposite.setLayoutData(data);
      Label warningLabel = new Label(warningComposite, SWT.NONE);
      warningLabel.setImage(Dialog.getImage(Dialog.DLG_IMG_MESSAGE_WARNING));
      warningLabel.setLayoutData(new GridData(GridData.VERTICAL_ALIGN_BEGINNING | GridData.HORIZONTAL_ALIGN_BEGINNING));
      Label warningText = new Label(warningComposite, SWT.WRAP);
      warningText.setText(message); 
      data = new GridData(GridData.FILL_HORIZONTAL);
      data.widthHint = 300;
      GC gc= new GC(composite);
      gc.setFont(parent.getFont());
      FontMetrics fontMetrics= gc.getFontMetrics();
      gc.dispose();
      data.heightHint= Dialog.convertHeightInCharsToPixels(fontMetrics, 3);
      warningText.setLayoutData(data);
  }
  
  /** Creates the root panel for the editor. <BR>
   */
  private Composite createRootPanel() {
    Composite result;
    GridLayout layout;
    GridData gridData;
    
    layout = new GridLayout();
    layout.numColumns = 1;
    parent.setLayout(layout); 
    gridData = new GridData(GridData.FILL_BOTH);
    //gridData.grabExcessHorizontalSpace = true;
    //gridData.grabExcessVerticalSpace = true;
    gridData.widthHint = 300; 
    result = new Composite(parent, SWT.NULL);
    layout = new GridLayout();
    layout.numColumns = 1; 
    result.setLayout(layout);
    result.setLayoutData(gridData);
    return result;
  }
  
  /** Gets the database properties with the name passed. <BR>
   *  @param name the name of the properties
   *  @return the properties object or <code>null</code>
   */
  private DatabaseProperties getDriver(String name) {
    DatabaseProperties properties;
    
    for (int counter = 0; counter < drivers.size(); counter++) {
      properties = (DatabaseProperties) drivers.get(counter);
      if (properties.getName().equals(name)) {
        return properties;
      }
    }
    return null;
  }
  
  /** Creates the driver combo. <BR>
   */
  private void createDriverCombo(Composite parent) {
    DatabaseProperties properties;
    GridData gridData;
    Composite container;
    GridLayout layout;
    Label label;
    
    container = new Composite(parent, SWT.NULL);
    layout = new GridLayout();
    layout.numColumns = 2;
    layout.marginWidth = 0;
    layout.marginHeight = 0;
    container.setLayout(layout);
    label = new Label(container, SWT.NULL);
    label.setText("Connection type:");
    gridData = new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING,
                            GridData.VERTICAL_ALIGN_CENTER, true, true);
    gridData.horizontalSpan = 2;
    container.setLayoutData(gridData);
    gridData = new GridData(GridData.HORIZONTAL_ALIGN_END,
                            GridData.VERTICAL_ALIGN_BEGINNING, true, true);
    
    label.setLayoutData(gridData);
    driverCombo = new Combo(container, SWT.READ_ONLY);
    gridData = new GridData(GridData.GRAB_HORIZONTAL | GridData.HORIZONTAL_ALIGN_FILL | GridData.VERTICAL_ALIGN_FILL);
    gridData.horizontalSpan = 1;
    driverCombo.setLayoutData(gridData);
    drivers = JdbcPlugin.getDefault().getDatabaseProperties();
    for (int counter = 0; counter < drivers.size(); counter++) {
      properties = (DatabaseProperties) drivers.get(counter);
      driverCombo.add(properties.getName());
    }
    if (drivers.size() > 0) {
      driverCombo.setText(((DatabaseProperties) drivers.get(0)).getName());
    } else {
      driverCombo.setEnabled(false);
    }
  }
  
  private Composite createGroupContainer(Group parent) {
      Composite container = new Composite(parent, SWT.NULL);
      GridLayout layout = new GridLayout();
      layout.numColumns = 2;
      //layout.marginWidth = 0;
      container.setLayout(layout);
      GridData gridData = new GridData(GridData.FILL_BOTH);
      gridData.horizontalSpan = 2;
      container.setLayoutData(gridData);
      return container;
    }

  
  /** Gets the current settings of the fields. <BR>
   */
  public DBConnectionProperties getCurrentSettings() {
    DBConnectionProperties result;
    
    result = new DBConnectionProperties(nameField.getStringValue(),
                                        urlField.getStringValue(),
                                        userNameField.getStringValue(),
                                        passwordField.getStringValue(),
                                        getDriver(driverCombo.getText()));
    return result;
  }
  
  /** Tests if the editor is in edit connection mode. <BR>
   */
  public boolean inEditMode() {
    return mode == MODE_EDIT;
  }
  
  /** Tests if the editor is in new connection mode. <BR>
   */
  public boolean inNewMode() { 
    return mode == MODE_NEW;
  }
  
  /** Validates the current settings. <BR>
   *  @return an error message or <code>null</code> if everything is okay
   */
  public Vector validateSettings() {
      if (wizardPage == null) return new Vector();
      String pluginID = DbPlugin.ID;
      Vector messages = new Vector();
      if ((nameField.getStringValue() == null) || (nameField.getStringValue().length() == 0)) {
          messages.add(new Status(IStatus.OK, pluginID, 0, "Please enter a name for the location.", null));
      } else if (inNewMode() && DbPlugin.getDefault().containsProperties(nameField.getStringValue())) {
          messages.add(new Status(IStatus.ERROR, pluginID, 0, "A location with this name already exists.", null));
      }
      if ((urlField.getStringValue() == null) || (urlField.getStringValue().length() == 0)) {
          messages.add(new Status(IStatus.OK, pluginID, 0, "Please enter an url for the location.", null));
      } 
      messages.add(new Status(IStatus.OK, pluginID, 0, wizardPage.getTitle(), null));
      return messages;
  }

	/**
	 * Applies the status to the status line of a dialog page.
	 */
	private void applyToStatusLine(Vector v) {
	    if (wizard!=null) {
	        wizard.getContainer().updateButtons();
	    }
	    if (wizardPage == null) return;
	    if (v.size() == 0) {
			wizardPage.setErrorMessage(null);
			wizardPage.setMessage(null);
	    }
	    IStatus status = (IStatus) v.get(0);
		String message= status.getMessage();
		if (message.length() == 0) message= null;
		switch (status.getSeverity()) {
			case IStatus.OK:
				wizardPage.setErrorMessage(null);
				wizardPage.setMessage(message);
				break;
			case IStatus.WARNING:
			    wizardPage.setErrorMessage(null);
				wizardPage.setMessage(message, IMessageProvider.WARNING);
				break;				
			case IStatus.INFO:
			    wizardPage.setErrorMessage(null);
				wizardPage.setMessage(message, IMessageProvider.INFORMATION);
				break;			
			default:
			    wizardPage.setErrorMessage(message);
				wizardPage.setMessage(null);
				break;		
		}
	}	

	

	/**
	 * @param wizard
	 */
	public void setWizard(IWizard wizard) {
	    this.wizard = wizard;
	}

    /**
     * @param page
     */
    public void setWizardPage(WizardPage page) {
        this.wizardPage = page;
        
    }
}
